//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
/** \file vm/virtual_machine.hh
    \brief Declare Virtual Machine class. */
#ifndef VM_VIRTUAL_MACHINE_HH
# define VM_VIRTUAL_MACHINE_HH

# include <ostream>

# include "vm/fwd.hh"
# include "vm/memory.hh"
# include "vm/mmu.hh"
# include "vm/cp0.hh"
# include "vm/cpu.hh"

namespace inst { class Program; }

namespace vm
{

  /// Virtual Machine abstraction
  class VirtualMachine
  {
  public:
    enum mode_type
      {
        normal,
        step
      };

    enum status_type
      {
        run,
        pause,
        stop,
        halt
      };

    /** \name Constructor and destructor.
        \{ */
  public:
    VirtualMachine(mode_type mode = normal);
    /** \} */

  public:
    /// Reset the vm
    void reset();

    /// Print the vm
    void write(std::ostream& ostr) const;

  public:
    /// Load a program into memory.
    void load_program(const inst::Program &program);

  public:
    /// Execute a program.
    void execute();

    /** \name Shell entry point
        \{ */
  public:
    /// Execute a program
    void execute(bool trace_p);

    /// Add a breakpoint
    void add_breakpoint(int offset);

    /// Return CPU's call stack
    const std::vector<register_type> &get_call_stack() const;

  public:
    /// Make the vm runnable
    void set_runnable();

    /// Run only the next instruction
    void set_run_next();

    /// Return the status of the vm
    status_type get_status() const;

    const Cp0 &get_cp0() const;
    Cp0 &get_cp0();

    const Cpu &get_cpu() const;
    Cpu &get_cpu();

  protected:
    /// The virtual machine mode.
    mode_type   mode_;
    /// The virtual machine status.
    status_type status_;
    /** \} */

  protected:
    int main_offset_; // FIXME: Must disappear!

  protected:
    /// The list of breakpoints
    std::list<int> breakpoints_; // FIXME: Should be in dedicated hardware!

  protected:
    /// Memory block.
    Memory memory_;

    /// Memory management unit.
    Mmu mmu_;
    /// Control coprocessor.
    Cp0 cp0_;
    /// Central processor unit.
    Cpu cpu_;
  };

  inline std::ostream&
  operator<<(std::ostream &os, const VirtualMachine &vm);

} // namespace vm

# include "vm/virtual_machine.hxx"

#endif // !VM_VIRTUAL_MACHINE_HH
